"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.normalizer = normalizer;
exports.denormalizer = denormalizer;
exports.normalize = normalize;
exports.denormalize = denormalize;
function normalizer(array) {
    var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'id';


    if (!Array.isArray(array)) {
        return array;
    }

    if (!array.length) {
        return {
            byId: {},
            ids: []
        };
    }

    return array.reduce(function (obj, item) {
        obj.ids = obj.ids || [];
        obj.byId = obj.byId || {};

        obj.ids.push(item[key]);
        obj.byId[item[key]] = item;

        return obj;
    }, {});
}

function denormalizer(object) {
    if ((typeof object === "undefined" ? "undefined" : _typeof(object)) !== "object") {
        return object;
    }
    return object.ids.map(function (id) {
        return object.byId[id];
    });
}

function normalize(data, schema) {
    if (Array.isArray(data)) {
        return data.map(function (item) {
            return normalize(item, schema);
        });
    } else {
        var result = _extends({}, data);
        Object.keys(schema).forEach(function (key) {
            if (!schema[key]) {
                return data;
            };

            var _schema$key = schema[key],
                childSchema = _schema$key.schema,
                childId = _schema$key.id;


            if (childSchema) {
                var child = normalize(data[key], childSchema);
                result[key] = normalizer(child, childId);
            } else {
                result[key] = normalizer(data[key], childId);
            }
        });

        return result;
    }
}

function denormalize(data, schema) {
    if (Array.isArray(data)) {
        return data.map(function (item) {
            return denormalize(item, schema);
        });
    } else {
        var result = _extends({}, data);
        Object.keys(schema).forEach(function (key) {
            if (!schema[key]) {
                return data;
            };
            var childSchema = schema[key].schema;

            if (childSchema) {
                result[key] = data[key].ids.map(function (id) {
                    return denormalize(data[key].byId[id], childSchema);
                });
            } else {
                result[key] = denormalizer(data[key]);
            }
        });

        return result;
    }
}